#ifndef SNORT_HPP
#define SNORT_HPP

#include <fstream>
#include <iostream>
#include <map>
#include <ctime>
#include <vector>
#include <boost/algorithm/string.hpp>
#include <sys/utsname.h>
#include <sstream>
#include <syslog.h>

#include <plugins/plugin.hpp>


#include <boost/extension/extension.hpp>
#include <boost/extension/factory.hpp>
#include <boost/extension/type_map.hpp>
#include <boost/property_tree/ptree.hpp>
#include <boost/property_tree/xml_parser.hpp>


#include <ndds/ndds_cpp.h>
#include "ids_dds.h"
#include "ids_ddsSupport.h"


struct ids_alert_without_timestamp {
	std::string sig_generator;
	std::string sig_id;
	std::string sig_rev;
	std::string msg;
	std::string proto;
	std::string src;
	std::string srcport;
	std::string dst;
	std::string dstport;
	std::string ethsrc;
	std::string ethdst;
	std::string ethlen;
	std::string tcpflags;
	std::string tcpseq;
	std::string tcpack;
	std::string tcplen;
	std::string tcpwindow;
	std::string ttl;
	std::string tos;
	std::string id;
	std::string dgmlen;
	std::string iplen;
	std::string icmptype;
	std::string icmpcode;
	std::string icmpid;
	std::string icmpseq;
};

struct date {
	int mday;
	int mon;
	int hour;
	int min;
	int sec;
};

class snort : public plugin {
public:
	snort(DDSDomainParticipant *participant);
	virtual void publish_information(void);
private:
	void update_alertsmap(void);
	void get_values(void);
	bool is_newer(std::string timestamp);
	date get_date_from_timestamp(std::string timestamp);
	date latest_date_;
	std::map<std::string, struct ids_alert_without_timestamp> alert_map_;
	std::string qos_file_;
    std::string log_file_;
    host_info hi_;
    	
	//DDS_Stuff
	DDSDomainParticipant *participant_;
	DDSPublisher *publisher_;
	DDSTopic *topic_;
	DDSDataWriter *writer_;
	ids_alertDataWriter * ids_alert_writer_;
	ids_alert *instance_;
	DDS_ReturnCode_t retcode_;
	DDS_InstanceHandle_t instance_handle_;
	const char *type_name_;
	DDS_Duration_t send_period_;	
	int this_year_;
	

};


BOOST_EXTENSION_TYPE_MAP_FUNCTION
{
	using namespace boost::extensions;
	std::map<std::string, factory<plugin, DDSDomainParticipant *> > &plugin_factories(types.get());
	plugin_factories["snort"].set<snort>();
}

#endif //SNORT_HPP
